use std::fs;

use serde::{Deserialize, Serialize};
use sqlx::SqlitePool;
use thiserror::Error;
use tracing::{error, warn};

#[derive(Debug, Clone, Serialize, Deserialize)]
pub struct Config {
    pub host: String,
    pub port: u16,
    pub videos_per_page: usize,
}

impl Default for Config {
    fn default() -> Self {
        Self {
            host: "0.0.0.0".into(),
            port: 3000,
            videos_per_page: 10,
        }
    }
}

/// This controls an instance and its state.
#[derive(Debug, Clone)]
pub struct Instance {
    pub config: Config,
    pub key_hash: String,
    pub pool: SqlitePool,
}

#[derive(Debug, Error)]
pub enum NewInstanceError {
    #[error("Failed to open TOML configuration file")]
    ConfigLoad(#[from] std::io::Error),
    #[error("Could not parse TOML configuration")]
    ConfigParse(#[from] toml::de::Error),
    #[error("Failed to create connection pool")]
    PoolConnect(#[from] sqlx::Error),
    #[error("Almond API key hash missing from environment variables")]
    MissingHashEnv,
}

impl Instance {
    pub async fn new() -> Result<Self, NewInstanceError> {
        let default_database_url = "sqlite://app.db".into();
        let database_url =
            std::env::var("DATABASE_URL").map_err(|_| {
                warn!("Could not find DATABASE_URL environment variable! Using default '{default_database_url}'");
            }).unwrap_or(default_database_url);

        let config = toml::from_str(&fs::read_to_string("almond.toml")?)?;

        let key_hash = std::env::var("INSTANCE_KEY_HASH").map_err(|_| {
            error!("Could not find INSTANCE_KEY_HASH environment variable!");
            NewInstanceError::MissingHashEnv
        })?;

        let pool = SqlitePool::connect(&database_url).await?;

        Ok(Self {
            config,
            key_hash,
            pool,
        })
    }
}
